# ID Broker plugins

## Intro
This repository contains a Debian source package for the ID Broker systems.
The following binary Debian packages are created from it:

* `id-broker-common`: scripts and code used in multiple other packages
* `ucs-test-id-broker-common`: tests for `id-broker-common`
* `id-broker-ldap-schema`: installation of LDAP schema, LDAP ACLs and extended UDM properties
* `id-broker-provisioning-api-plugin`: plugin for the `ucsschool-apis` app with the REST resources of the Provisioning API
* `id-broker-self-disclosure-api-plugin`: plugin for the `ucsschool-apis` app with the REST resources of the Self-Disclosure API

The Provisioning API and the Self-Disclosure APIs connect to the Kelvin REST API, the UDM REST API and OpenLDAP (directly) to CRUD users, groups and OUs.
The setup of those connections is configured through join scripts.

High level and also in-depth information on the ID Broker project can be found in the following documentation:

* Architecture documentation: https://univention.gitpages.knut.univention.de/id-broker/architecture-documentation/
* Operations manual (for Linet): https://univention.gitpages.knut.univention.de/id-broker/operations-manual/
* Documentation for school authorities: *TODO* -> https://git.knut.univention.de/univention/ucsschool/-/issues/352
* Documentation for service providers: *TODO* -> https://git.knut.univention.de/univention/ucsschool/-/issues/506

## Installation & configuration

Installation instructions for the *Self-Disclosure API* can be found here: https://univention.gitpages.knut.univention.de/id-broker/operations-manual/installation.html#backup-self-disclosure-api

Installation instructions for the *Provisioning API* can be found here: https://univention.gitpages.knut.univention.de/id-broker/operations-manual/installation.html#backup-provisioning-api

### Kelvin REST API

The Kelvin REST API will be configured by the join scripts to include the extended properties for the service provider specific pseudonyms (in `idBrokerPseudonym____`), the source objects `entryUUID` (in `ucsschoolRecordUID`), and the school authority (in `ucsschoolSourceUID`) it originates from.
The extended properties mapping configuration is in `/etc/ucsschool/kelvin/mapped_udm_properties.json`.

Additionally, the join scripts configure Kelvin to accept usernames with a length of up to `50` characters.
This is configured on `/var/lib/ucs-school-import/configs/kelvin.json`.

### Provisioning API and Self-Disclosure API configuration

It is assumed that the Provisioning API and the Self-Disclosure API are installed in different systems (UCS `backup` role).

The Provisioning API and Self-Disclosure API plugins for the `ucsschool-apis` apps are enabled in `/etc/ucsschool/apis/settings.json`.
This is also where their authentication strategies (provided by the `auth` plugin) are configured.

The APIs access the Kelvin REST API using the `ucsschool-apis` apps machine account.
The credentials are stored in `/etc/ucsschool/apis/id-broker/provisioning-api.json` and `/etc/ucsschool/apis/id-broker/self-disclosure-api.json` respectively.

## Swagger UI

Both REST APIs can be accessed interactively through the Swagger UI at https://FQDN/ucsschool/apis/docs

## Performance tests

Performance tests for the Provisioning API and the Self disclosure API are done using Locust (http://docs.locust.io/).

Locust files are located in the `performance-tests/performance_tests` directory.
Test data is located in `performance-tests/data`.

The Locust files and test data can be used directly from git or can be installed by a Debian package.

To use them **from git**, create a virtual env and install the `setup.py`:

```bash
python3 -m venv venv
. venv/bin/activate
pip install -U pip wheel
pip install -e performance-tests
```

Then (after installing the test data - see `Test data for performance tests` below) execute for example:

```bash
locust --locustfile performance-tests/performance_tests/provisioning/school_class_get.py
```

To run the performance tests **from UCS 5.0**, install the package `id-broker-performance-tests` and (after installing the test data) execute:

```bash
BASE_DIR="/var/lib/id-broker-performance-tests"
$BASE_DIR/venv/bin/locust --locustfile $BASE_DIR/performance_tests/provisioning/school_head.py
```

### Locust commandline parameters

The default log level is `INFO`. To see debug messages, add to the command line: `--loglevel DEBUG`

To run it non-interactively add for example:

```
--host 10.200.3.83 --headless --users 10 --spawn-rate 1 --run-time 5m --stop-timeout 60 --csv=id_broker_school_class_get --html=id_broker_school_class_get.html --print-stats --exit-code-on-error 0
```

For more information see http://docs.locust.io/en/stable/running-without-web-ui.html

When Locust is started, it opens a web server at `http://0.0.0.0:8089`.
See http://docs.locust.io/en/stable/quickstart.html how to use it.

If one client CPU does not generate enough load on the server, see http://docs.locust.io/en/stable/running-distributed.html

### Test data for performance tests

The performance tests require knowledge of school, class and user objects and their attributes, existing on the ID Broker and school authority systems.
That data is provided in the form of LDIF, JSON and SQLite files.

For more information about the creation of the LDIF, JSON and SQLite files see the respective [README](performance-tests/prepare_ldap/README.md).
